"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const YAML = __importStar(require("js-yaml"));
const fs = __importStar(require("fs"));
const api_1 = __importDefault(require("@mesg/api"));
const bs58 = __importStar(require("@mesg/api/lib/util/base58"));
const encoder_1 = require("@mesg/api/lib/util/encoder");
const types_1 = require("@mesg/api/lib/types");
class Service {
    constructor(options = {}) {
        this.definition = options.definition || YAML.safeLoad(fs.readFileSync('./mesg.yml').toString());
        this.API = options.API || new api_1.default(process.env.MESG_ENDPOINT);
        this.runnerHash = options.runnerHash || bs58.decode(process.env.MESG_RUNNER_HASH);
        this.instanceHash = options.instanceHash || bs58.decode(process.env.MESG_INSTANCE_HASH);
    }
    listenTask(_a) {
        var tasks = __rest(_a, []);
        if (this.tasks) {
            throw new Error(`listenTask should be called only once`);
        }
        this.tasks = tasks;
        this.validateTaskNames();
        const stream = this.API.execution.stream({
            filter: {
                executorHash: this.runnerHash,
                statuses: [types_1.ExecutionStatus.IN_PROGRESS],
            }
        });
        stream.on('data', this.handleTaskData.bind(this));
        return stream;
    }
    emitEvent(event, data) {
        if (!data)
            throw new Error('data object must be send while emitting event');
        return this.API.event.create({
            instanceHash: this.instanceHash,
            key: event,
            data: encoder_1.encode(data)
        });
    }
    handleTaskData({ hash, taskKey, inputs }) {
        return __awaiter(this, void 0, void 0, function* () {
            const callback = this.tasks[taskKey];
            if (!callback) {
                throw new Error(`Task ${taskKey} is not defined in your services`);
            }
            try {
                const outputs = yield callback(encoder_1.decode(inputs));
                return this.API.execution.update({
                    hash,
                    outputs: encoder_1.encode(outputs)
                });
            }
            catch (err) {
                const error = err.message;
                return this.API.execution.update({ hash, error });
            }
        });
    }
    validateTaskNames() {
        const nonDescribedTasks = Object.keys(this.tasks).filter(x => !this.definition.tasks[x]);
        if (nonDescribedTasks.length > 0) {
            throw new Error(`The following tasks are not present in the mesg.yml: ${nonDescribedTasks.join(', ')}`);
        }
        const nonHandledTasks = Object.keys(this.definition.tasks).filter(x => !this.tasks[x]);
        if (nonHandledTasks.length > 0) {
            console.warn(`WARNING: The following tasks described in the mesg.yml haven't been implemented: ${nonHandledTasks.join(', ')}`);
        }
    }
}
exports.default = Service;
(module).exports = Service;
