"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const tape_1 = __importDefault(require("tape"));
const sinon = __importStar(require("sinon"));
const _1 = __importDefault(require("."));
const mock_1 = __importDefault(require("@mesg/api/lib/mock"));
const encoder_1 = require("@mesg/api/lib/util/encoder");
const runnerHash = Buffer.from("runnerHash");
const instanceHash = Buffer.from("instanceHash");
function newService({ definition = {}, api = mock_1.default(''), }) {
    return new _1.default({
        runnerHash,
        instanceHash,
        definition,
        API: api
    });
}
tape_1.default('listenTask() should pass task validation', function (t) {
    t.plan(1);
    const definition = { tasks: { "task1": {}, "task2": {} } };
    const service = newService({ definition });
    sinon.stub(service, 'listenTask');
    try {
        service.listenTask({ "task1": () => ({}), "task2": () => ({}) });
        t.pass("tasks valid");
    }
    catch (e) {
        t.error(e);
    }
});
tape_1.default('listenTask() should throw because missing task in mesg.yml', function (t) {
    t.plan(1);
    const definition = { tasks: { "task1": {} } };
    const service = newService({ definition });
    try {
        service.listenTask({ "task1": () => ({}), "task2": () => ({}) });
        t.fail("should throw");
    }
    catch (e) {
        t.ok(e);
    }
});
tape_1.default('listenTask() should give warning because missing task callback', function (t) {
    t.plan(1);
    const definition = { tasks: { "task1": {}, "task2": {} } };
    const service = newService({ definition });
    const spy = sinon.spy(console, 'warn');
    service.listenTask({ "task1": () => ({}) });
    t.equal(spy.getCall(0).args[0], 'WARNING: The following tasks described in the mesg.yml haven\'t been implemented: task2');
    spy.restore();
});
tape_1.default('listenTask() should throw when called more than once', function (t) {
    t.plan(1);
    const definition = { tasks: { "task1": {} } };
    const service = newService({ definition });
    service.listenTask({ "task1": () => ({}) });
    try {
        service.listenTask({ "task1": () => ({}) });
        t.fail("should throw");
    }
    catch (e) {
        t.ok(e);
    }
});
tape_1.default('listenTask() should listen for tasks', function (t) {
    t.plan(2);
    const api = mock_1.default('');
    const spy = sinon.spy(api.execution, 'stream');
    const definition = { tasks: { 'task1': {}, 'task2': {} } };
    const service = newService({ definition, api });
    service.listenTask({ 'task1': () => ({}), 'task2': () => ({}) });
    t.ok(spy.calledOnce);
    t.equal(spy.getCall(0).args[0].filter.executorHash, runnerHash);
    spy.restore();
});
tape_1.default('listenTask() should handle tasks and submit result', function (t) {
    return __awaiter(this, void 0, void 0, function* () {
        t.plan(4);
        const hash = 'hash';
        const inputs = { input: 'data' };
        const outputs = { output: 'data' };
        const api = mock_1.default('');
        const spy = sinon.spy(api.execution, 'update');
        const definition = { tasks: { 'task1': { inputs: {}, outputs: {} } } };
        const service = newService({ definition, api });
        const stream = service.listenTask({
            'task1': (taskinputs) => {
                t.deepEqual(taskinputs, inputs);
                return outputs;
            }
        });
        t.doesNotThrow(() => stream.emit('data', { hash, taskKey: 'task1', inputs: encoder_1.encode(inputs) }));
        yield setTimeout(() => { }, 0);
        const args = spy.getCall(0).args[0];
        spy.restore();
        t.equal(args.hash, hash);
        t.equal(JSON.stringify(args.outputs), JSON.stringify(encoder_1.encode(outputs)));
    });
});
tape_1.default('emitEvent() should emit an event', function (t) {
    t.plan(4);
    const key = 'event1';
    const data = { event: 'data' };
    const api = mock_1.default('');
    const spy = sinon.spy(api.event, 'create');
    const service = newService({ api });
    t.doesNotThrow(() => service.emitEvent(key, data));
    const args = spy.getCall(0).args[0];
    t.equal(args.instanceHash, instanceHash);
    t.equal(args.key, key);
    t.equal(JSON.stringify(args.data), JSON.stringify(encoder_1.encode(data)));
    spy.restore();
});
tape_1.default('emitEvent() should throw when no data provided', function (t) {
    t.plan(1);
    const service = newService({});
    try {
        service.emitEvent('event1');
    }
    catch (e) {
        t.equal(e.message, 'data object must be send while emitting event');
    }
});
